@@ -7,26 +7,11 @@
  ******************************************************************************
  * Copyright (c) 2021, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_json.h"
+#include "cpl_http.h"
 #include "vrtdataset.h"
 #include "ogr_spatialref.h"
 
@@ -59,7 +44,7 @@ struct AssetSetByProjection
 struct Asset
 {
     std::string osName{};
-    CPLJSONArray eoBands{};
+    CPLJSONArray bands{};
     std::map<std::string, AssetSetByProjection> assets{};
 };
 
@@ -97,6 +82,7 @@ class STACITDataset final : public VRTDataset
 
 STACITDataset::STACITDataset() : VRTDataset(0, 0)
 {
+    poDriver = nullptr;  // cancel what the VRTDataset did
     SetWritable(false);
 }
 
@@ -111,6 +97,13 @@ int STACITDataset::Identify(GDALOpenInfo *poOpenInfo)
         return true;
     }
 
+    const bool bIsSingleDriver = poOpenInfo->IsSingleAllowedDriver("STACIT");
+    if (bIsSingleDriver && (STARTS_WITH(poOpenInfo->pszFilename, "http://") ||
+                            STARTS_WITH(poOpenInfo->pszFilename, "https://")))
+    {
+        return true;
+    }
+
     if (poOpenInfo->nHeaderBytes == 0)
     {
         return false;
@@ -122,10 +115,27 @@ int STACITDataset::Identify(GDALOpenInfo *poOpenInfo)
         // before the loop.
         const char *pszHeader =
             reinterpret_cast<const char *>(poOpenInfo->pabyHeader);
-        if (strstr(pszHeader, "\"stac_version\"") != nullptr &&
-            strstr(pszHeader, "\"proj:transform\"") != nullptr)
+        while (*pszHeader != 0 &&
+               std::isspace(static_cast<unsigned char>(*pszHeader)))
+            ++pszHeader;
+        if (bIsSingleDriver)
         {
-            return true;
+            return pszHeader[0] == '{';
+        }
+
+        if (strstr(pszHeader, "\"stac_version\"") != nullptr)
+        {
+            int nTransformBBOXShapeCount = 0;
+            for (const char *pszItem :
+                 {"\"proj:transform\"", "\"proj:bbox\"", "\"proj:shape\""})
+            {
+                if (strstr(pszHeader, pszItem))
+                    nTransformBBOXShapeCount++;
+            }
+            if (nTransformBBOXShapeCount >= 2)
+            {
+                return true;
+            }
         }
 
         if (i == 0)
@@ -161,7 +171,7 @@ static std::string SanitizeCRSValue(const std::string &v)
         }
     }
     if (!ret.empty() && ret.back() == '_')
-        ret.resize(ret.size() - 1);
+        ret.pop_back();
     return ret;
 }
 
@@ -217,34 +227,44 @@ static void ParseAsset(const CPLJSONObject &jAsset,
         return oProperties[pszName];
     };
 
-    auto oProjEPSG = GetAssetOrFeatureProperty("proj:epsg");
     std::string osProjUserString;
-    if (oProjEPSG.IsValid() && oProjEPSG.GetType() != CPLJSONObject::Type::Null)
+    const auto oProjCode = GetAssetOrFeatureProperty("proj:code");
+    if (oProjCode.IsValid() && oProjCode.GetType() != CPLJSONObject::Type::Null)
     {
-        osProjUserString = "EPSG:" + oProjEPSG.ToString();
+        osProjUserString = oProjCode.ToString();
     }
     else
     {
-        auto oProjWKT2 = GetAssetOrFeatureProperty("proj:wkt2");
-        if (oProjWKT2.IsValid() &&
-            oProjWKT2.GetType() == CPLJSONObject::Type::String)
+        const auto oProjEPSG = GetAssetOrFeatureProperty("proj:epsg");
+        if (oProjEPSG.IsValid() &&
+            oProjEPSG.GetType() != CPLJSONObject::Type::Null)
         {
-            osProjUserString = oProjWKT2.ToString();
+            osProjUserString = "EPSG:" + oProjEPSG.ToString();
         }
         else
         {
-            auto oProjPROJJSON = GetAssetOrFeatureProperty("proj:projjson");
-            if (oProjPROJJSON.IsValid() &&
-                oProjPROJJSON.GetType() == CPLJSONObject::Type::Object)
+            const auto oProjWKT2 = GetAssetOrFeatureProperty("proj:wkt2");
+            if (oProjWKT2.IsValid() &&
+                oProjWKT2.GetType() == CPLJSONObject::Type::String)
             {
-                osProjUserString = oProjPROJJSON.ToString();
+                osProjUserString = oProjWKT2.ToString();
             }
             else
             {
-                CPLDebug("STACIT",
-                         "Skipping asset %s that lacks a valid CRS member",
-                         osAssetName.c_str());
-                return;
+                const auto oProjPROJJSON =
+                    GetAssetOrFeatureProperty("proj:projjson");
+                if (oProjPROJJSON.IsValid() &&
+                    oProjPROJJSON.GetType() == CPLJSONObject::Type::Object)
+                {
+                    osProjUserString = oProjPROJJSON.ToString();
+                }
+                else
+                {
+                    CPLDebug("STACIT",
+                             "Skipping asset %s that lacks a valid CRS member",
+                             osAssetName.c_str());
+                    return;
+                }
             }
         }
     }
@@ -288,6 +308,10 @@ static void ParseAsset(const CPLJSONObject &jAsset,
         for (const auto &oItem : oProjTransform)
             transform.push_back(oItem.ToDouble());
         CPLAssert(transform.size() == 6 || transform.size() == 9);
+#if defined(__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wnull-dereference"
+#endif
         if (transform[0] <= 0 || transform[1] != 0 || transform[3] != 0 ||
             transform[4] >= 0 ||
             (transform.size() == 9 &&
@@ -300,6 +324,9 @@ static void ParseAsset(const CPLJSONObject &jAsset,
                 osAssetName.c_str());
             return;
         }
+#if defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
     }
 
     if (bIsBBOXValid && bIsShapeValid)
@@ -372,7 +399,9 @@ static void ParseAsset(const CPLJSONObject &jAsset,
     {
         Asset asset;
         asset.osName = osAssetName;
-        asset.eoBands = jAsset.GetArray("eo:bands");
+        asset.bands = jAsset.GetArray("bands");
+        if (!asset.bands.IsValid())
+            asset.bands = jAsset.GetArray("eo:bands");
 
         collection.assets[osAssetName] = std::move(asset);
     }
@@ -450,6 +479,12 @@ bool STACITDataset::SetupDataset(
     }
 
     // Set raster size
+    if (dfXRes == 0 || dfYRes == 0)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Invalid computed dataset dimensions");
+        return false;
+    }
     double dfXSize = std::round((dfXMax - dfXMin) / dfXRes);
     double dfYSize = std::round((dfYMax - dfYMin) / dfYRes);
     if (dfXSize <= 0 || dfYSize <= 0 || dfXSize > INT_MAX || dfYSize > INT_MAX)
@@ -550,6 +585,7 @@ bool STACITDataset::SetupDataset(
               });
 
     // Create VRT bands and add sources
+    bool bAtLeastOneBandHasNoData = false;
     for (int i = 0; i < poItemDS->GetRasterCount(); i++)
     {
         auto poItemBand = poItemDS->GetRasterBand(i + 1);
@@ -559,41 +595,43 @@ bool STACITDataset::SetupDataset(
         int bHasNoData = FALSE;
         const double dfNoData = poItemBand->GetNoDataValue(&bHasNoData);
         if (bHasNoData)
+        {
+            bAtLeastOneBandHasNoData = true;
             poVRTBand->SetNoDataValue(dfNoData);
+        }
 
         const auto eInterp = poItemBand->GetColorInterpretation();
         if (eInterp != GCI_Undefined)
             poVRTBand->SetColorInterpretation(eInterp);
 
         // Set band properties
-        if (asset.eoBands.IsValid() &&
-            asset.eoBands.Size() == poItemDS->GetRasterCount())
+        if (asset.bands.IsValid() &&
+            asset.bands.Size() == poItemDS->GetRasterCount())
         {
-            const auto &eoBand = asset.eoBands[i];
-            const auto osBandName = eoBand["name"].ToString();
+            const auto &band = asset.bands[i];
+            const auto osBandName = band["name"].ToString();
             if (!osBandName.empty())
                 poVRTBand->SetDescription(osBandName.c_str());
 
-            if (eInterp != GCI_Undefined)
+            auto osCommonName = band["eo:common_name"].ToString();
+            if (osCommonName.empty())
+                osCommonName = band["common_name"].ToString();
+            if (!osCommonName.empty())
             {
-                const auto osCommonName = eoBand["common_name"].ToString();
-                if (osCommonName == "red")
-                    poVRTBand->SetColorInterpretation(GCI_RedBand);
-                else if (osCommonName == "green")
-                    poVRTBand->SetColorInterpretation(GCI_GreenBand);
-                else if (osCommonName == "blue")
-                    poVRTBand->SetColorInterpretation(GCI_BlueBand);
-                else if (osCommonName == "alpha")
-                    poVRTBand->SetColorInterpretation(GCI_AlphaBand);
+                const auto eInterpFromCommonName =
+                    GDALGetColorInterpFromSTACCommonName(osCommonName.c_str());
+                if (eInterpFromCommonName != GCI_Undefined)
+                    poVRTBand->SetColorInterpretation(eInterpFromCommonName);
             }
 
-            for (const auto &eoBandChild : eoBand.GetChildren())
+            for (const auto &bandChild : band.GetChildren())
             {
-                const auto osChildName = eoBandChild.GetName();
-                if (osChildName != "name" && osChildName != "common_name")
+                const auto osChildName = bandChild.GetName();
+                if (osChildName != "name" && osChildName != "common_name" &&
+                    osChildName != "eo:common_name")
                 {
                     poVRTBand->SetMetadataItem(osChildName.c_str(),
-                                               eoBandChild.ToString().c_str());
+                                               bandChild.ToString().c_str());
                 }
             }
         }
@@ -627,9 +665,17 @@ bool STACITDataset::SetupDataset(
             }
         }
 
-        const char *apszOptions[] = {"EMIT_ERROR_IF_GEOS_NOT_AVAILABLE=NO",
-                                     nullptr};
-        poVRTBand->RemoveCoveredSources(apszOptions);
+        const char *pszOverlapStrategy =
+            CSLFetchNameValueDef(poOpenInfo->papszOpenOptions,
+                                 "OVERLAP_STRATEGY", "REMOVE_IF_NO_NODATA");
+        if ((EQUAL(pszOverlapStrategy, "REMOVE_IF_NO_NODATA") &&
+             !bAtLeastOneBandHasNoData) ||
+            EQUAL(pszOverlapStrategy, "USE_MOST_RECENT"))
+        {
+            const char *const apszOptions[] = {
+                "EMIT_ERROR_IF_GEOS_NOT_AVAILABLE=NO", nullptr};
+            poVRTBand->RemoveCoveredSources(apszOptions);
+        }
     }
     return true;
 }
@@ -728,7 +774,9 @@ bool STACITDataset::Open(GDALOpenInfo *poOpenInfo)
     GIntBig nMaxItems = CPLAtoGIntBig(CSLFetchNameValueDef(
         poOpenInfo->papszOpenOptions, "MAX_ITEMS", "1000"));
 
-    if (CSLFetchNameValue(poOpenInfo->papszOpenOptions, "MAX_ITEMS") == nullptr)
+    const bool bMaxItemsSpecified =
+        CSLFetchNameValue(poOpenInfo->papszOpenOptions, "MAX_ITEMS") != nullptr;
+    if (!bMaxItemsSpecified)
     {
         // If the URL includes a limit parameter, and it's larger than our
         // default MAX_ITEMS value, then increase the later to the former.
@@ -742,13 +790,70 @@ bool STACITDataset::Open(GDALOpenInfo *poOpenInfo)
     }
 
     auto osCurFilename = osFilename;
+    std::string osMethod = "GET";
+    CPLJSONObject oHeaders;
+    CPLJSONObject oBody;
+    bool bMerge = false;
+    int nLoops = 0;
     do
     {
+        ++nLoops;
+        if (nMaxItems > 0 && nLoops > nMaxItems)
+        {
+            break;
+        }
+
         CPLJSONDocument oDoc;
+
         if (STARTS_WITH(osCurFilename, "http://") ||
             STARTS_WITH(osCurFilename, "https://"))
         {
-            if (!oDoc.LoadUrl(osCurFilename, nullptr))
+            // Cf // Cf https://github.com/radiantearth/stac-api-spec/tree/release/v1.0.0/item-search#pagination
+            CPLStringList aosOptions;
+            if (oBody.IsValid() &&
+                oBody.GetType() == CPLJSONObject::Type::Object)
+            {
+                if (bMerge)
+                    CPLDebug("STACIT",
+                             "Ignoring 'merge' attribute from next link");
+                const std::string osPostContent =
+                    oBody.Format(CPLJSONObject::PrettyFormat::Pretty);
+                aosOptions.SetNameValue("POSTFIELDS", osPostContent.c_str());
+            }
+            aosOptions.SetNameValue("CUSTOMREQUEST", osMethod.c_str());
+            CPLString osHeaders;
+            if (!oHeaders.IsValid() ||
+                oHeaders.GetType() != CPLJSONObject::Type::Object ||
+                oHeaders["Content-Type"].ToString().empty())
+            {
+                osHeaders = "Content-Type: application/json";
+            }
+            if (oHeaders.IsValid() &&
+                oHeaders.GetType() == CPLJSONObject::Type::Object)
+            {
+                for (const auto &obj : oHeaders.GetChildren())
+                {
+                    osHeaders += "\r\n";
+                    osHeaders += obj.GetName();
+                    osHeaders += ": ";
+                    osHeaders += obj.ToString();
+                }
+            }
+            aosOptions.SetNameValue("HEADERS", osHeaders.c_str());
+            CPLHTTPResult *psResult =
+                CPLHTTPFetch(osCurFilename.c_str(), aosOptions.List());
+            if (!psResult)
+                return false;
+            if (!psResult->pabyData)
+            {
+                CPLHTTPDestroyResult(psResult);
+                return false;
+            }
+            const bool bOK = oDoc.LoadMemory(
+                reinterpret_cast<const char *>(psResult->pabyData));
+            // CPLDebug("STACIT", "Response: %s", reinterpret_cast<const char*>(psResult->pabyData));
+            CPLHTTPDestroyResult(psResult);
+            if (!bOK)
                 return false;
         }
         else
@@ -757,16 +862,24 @@ bool STACITDataset::Open(GDALOpenInfo *poOpenInfo)
                 return false;
         }
         const auto oRoot = oDoc.GetRoot();
-        const auto oFeatures = oRoot.GetArray("features");
+        auto oFeatures = oRoot.GetArray("features");
         if (!oFeatures.IsValid())
         {
-            CPLError(CE_Failure, CPLE_AppDefined, "Missing features");
-            return false;
+            if (oRoot.GetString("type") == "Feature")
+            {
+                oFeatures = CPLJSONArray();
+                oFeatures.Add(oRoot);
+            }
+            else
+            {
+                CPLError(CE_Failure, CPLE_AppDefined, "Missing features");
+                return false;
+            }
         }
         for (const auto &oFeature : oFeatures)
         {
             nItemIter++;
-            if (nItemIter > nMaxItems)
+            if (nMaxItems > 0 && nItemIter > nMaxItems)
             {
                 break;
             }
@@ -830,10 +943,9 @@ bool STACITDataset::Open(GDALOpenInfo *poOpenInfo)
                            oMapCollection);
             }
         }
-        if (nItemIter >= nMaxItems)
+        if (nMaxItems > 0 && nItemIter >= nMaxItems)
         {
-            if (CSLFetchNameValue(poOpenInfo->papszOpenOptions, "MAX_ITEMS") ==
-                nullptr)
+            if (!bMaxItemsSpecified)
             {
                 CPLError(CE_Warning, CPLE_AppDefined,
                          "Maximum number of items (" CPL_FRMT_GIB
@@ -851,20 +963,35 @@ bool STACITDataset::Open(GDALOpenInfo *poOpenInfo)
         }
 
         // Follow next link
+        // Cf https://github.com/radiantearth/stac-api-spec/tree/release/v1.0.0/item-search#pagination
         const auto oLinks = oRoot.GetArray("links");
         if (!oLinks.IsValid())
             break;
         std::string osNewFilename;
         for (const auto &oLink : oLinks)
         {
-            if (oLink["rel"].ToString() == "next")
+            const auto osType = oLink["type"].ToString();
+            if (oLink["rel"].ToString() == "next" &&
+                (osType.empty() || osType == "application/geo+json"))
             {
+                osMethod = oLink.GetString("method", "GET");
                 osNewFilename = oLink["href"].ToString();
-                break;
+                oHeaders = oLink["headers"];
+                oBody = oLink["body"];
+                bMerge = oLink.GetBool("merge", false);
+                if (osType == "application/geo+json")
+                {
+                    break;
+                }
             }
         }
-        if (!osNewFilename.empty() && osNewFilename != osCurFilename)
+        if (!osNewFilename.empty() &&
+            (osNewFilename != osCurFilename ||
+             (oBody.IsValid() &&
+              oBody.GetType() == CPLJSONObject::Type::Object)))
+        {
             osCurFilename = osNewFilename;
+        }
         else
             osCurFilename.clear();
     } while (!osCurFilename.empty());
@@ -940,6 +1067,14 @@ void GDALRegister_STACIT()
         "       <Value>HIGHEST</Value>"
         "       <Value>LOWEST</Value>"
         "   </Option>"
+        "   <Option name='OVERLAP_STRATEGY' type='string-select' "
+        "default='REMOVE_IF_NO_NODATA' "
+        "description='Strategy to use when some sources are fully "
+        "covered by others'>"
+        "       <Value>REMOVE_IF_NO_NODATA</Value>"
+        "       <Value>USE_ALL</Value>"
+        "       <Value>USE_MOST_RECENT</Value>"
+        "   </Option>"
         "</OpenOptionList>");
 
     poDriver->pfnOpen = STACITDataset::OpenStatic;
